@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2017-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -22,8 +22,7 @@
  *
  */
 
-#ifndef PLUGIN_MANAGER_H
-#define PLUGIN_MANAGER_H
+#pragma once
 
 #include <QObject>
 
@@ -34,7 +33,7 @@ class VEYON_CORE_EXPORT PluginManager : public QObject
 {
 	Q_OBJECT
 public:
-	PluginManager( QObject* parent = nullptr );
+	explicit PluginManager( QObject* parent = nullptr );
 
 	void loadPlatformPlugins();
 	void loadPlugins();
@@ -75,5 +74,3 @@ signals:
 	void pluginsLoaded();
 
 };
-
-#endif // PLUGIN_MANAGER_H
