@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2017-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -32,7 +32,7 @@ CryptoCore::CryptoCore() :
 {
 	const auto features = QCA::supportedFeatures();
 
-	qDebug() << "CryptoCore instance created - features supported by QCA" << qcaVersionStr() << features;
+	vDebug() << "CryptoCore instance created - features supported by QCA" << qcaVersionStr() << features;
 
 	if( features.contains( QStringLiteral( "rsa" ) ) == false )
 	{
@@ -40,14 +40,14 @@ CryptoCore::CryptoCore() :
 				"(e.g. packages such as libqca-qt5-2-plugins or qca-qt5-ossl)." );
 	}
 
-	m_defaultPrivateKey = PrivateKey::fromPEMFile( QStringLiteral(":/resources/default-pkey.pem") );
+	m_defaultPrivateKey = PrivateKey::fromPEMFile( QStringLiteral(":/core/default-pkey.pem") );
 }
 
 
 
 CryptoCore::~CryptoCore()
 {
-	qDebug( "CryptoCore instance destroyed" );
+	vDebug();
 }
 
 
@@ -58,7 +58,7 @@ QByteArray CryptoCore::generateChallenge()
 
 	if( challengeBigNum == nullptr )
 	{
-		qCritical( "CryptoCore::generateChallenge(): BN_new() failed" );
+		vCritical() << "BN_new() failed";
 		return QByteArray();
 	}
 
@@ -75,7 +75,8 @@ QByteArray CryptoCore::generateChallenge()
 
 QString CryptoCore::encryptPassword( const QString& password ) const
 {
-	return m_defaultPrivateKey.toPublicKey().encrypt( password.toUtf8(), DefaultEncryptionAlgorithm ).toByteArray().toHex();
+	return QString::fromLatin1( m_defaultPrivateKey.toPublicKey().
+								encrypt( password.toUtf8(), DefaultEncryptionAlgorithm ).toByteArray().toHex() );
 }
 
 
@@ -87,10 +88,10 @@ QString CryptoCore::decryptPassword( const QString& encryptedPassword ) const
 	if( PrivateKey( m_defaultPrivateKey ).decrypt( QByteArray::fromHex( encryptedPassword.toUtf8() ),
 												   &decryptedPassword, DefaultEncryptionAlgorithm ) )
 	{
-		return decryptedPassword.toByteArray();
+		return QString::fromUtf8( decryptedPassword.toByteArray() );
 	}
 
-	qCritical("CryptoCore: failed to decrypt password!" );
+	vCritical() << "failed to decrypt password!";
 
-	return QString();
+	return {};
 }
