@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2017-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -23,7 +23,6 @@
  */
 
 #include <QCoreApplication>
-#include <QDebug>
 #include <QDir>
 #include <QPluginLoader>
 
@@ -74,7 +73,7 @@ void PluginManager::upgradePlugins()
 		const auto currentPluginVersion = pluginInterface->version();
 		if( currentPluginVersion > previousPluginVersion )
 		{
-			qDebug() << "Upgrading plugin" << pluginInterface->name()
+			vDebug() << "Upgrading plugin" << pluginInterface->name()
 					 << "from" << previousPluginVersion.toString()
 					 << "to" << currentPluginVersion.toString();
 			pluginInterface->upgrade( previousPluginVersion );
@@ -143,7 +142,7 @@ QString PluginManager::pluginName( Plugin::Uid pluginUid ) const
 		}
 	}
 
-	return QString();
+	return {};
 }
 
 
@@ -156,9 +155,10 @@ void PluginManager::initPluginSearchPath()
 		const auto pluginSearchPath = dir.absolutePath();
 		if( m_noDebugMessages == false )
 		{
-			qDebug() << "Adding plugin search path" << pluginSearchPath;
+			vDebug() << "Adding plugin search path" << pluginSearchPath;
 		}
 		QDir::addSearchPath( QStringLiteral( "plugins" ), pluginSearchPath );
+		QCoreApplication::addLibraryPath( pluginSearchPath );
 	}
 }
 
@@ -166,18 +166,27 @@ void PluginManager::initPluginSearchPath()
 
 void PluginManager::loadPlugins( const QString& nameFilter )
 {
-	auto plugins = QDir( QStringLiteral( "plugins:" ) ).entryInfoList( { nameFilter } );
+	const auto plugins = QDir( QStringLiteral( "plugins:" ) ).entryInfoList( { nameFilter } );
 	for( const auto& fileInfo : plugins )
 	{
+		const auto fileName = fileInfo.fileName();
+
+		// skip simple shared libraries
+		if( fileName.startsWith( QLatin1String("lib") ) &&
+			fileName.startsWith( QLatin1String("libveyon") ) == false )
+		{
+			continue;
+		}
+
 		auto pluginObject = QPluginLoader( fileInfo.filePath() ).instance();
 		auto pluginInterface = qobject_cast<PluginInterface *>( pluginObject );
 
 		if( pluginObject && pluginInterface &&
-				m_pluginInterfaces.contains( pluginInterface ) == false )
+			m_pluginInterfaces.contains( pluginInterface ) == false )
 		{
 			if( m_noDebugMessages == false )
 			{
-				qDebug() << "PluginManager: discovered plugin" << pluginInterface->name() << "at" << fileInfo.filePath();
+				vDebug() << "discovered plugin" << pluginInterface->name() << "at" << fileInfo.filePath();
 			}
 			m_pluginInterfaces += pluginInterface;	// clazy:exclude=reserve-candidates
 			m_pluginObjects += pluginObject;		// clazy:exclude=reserve-candidates
