@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2006-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -34,9 +34,12 @@
 #include <QHostAddress>
 #include <QLabel>
 #include <QProcessEnvironment>
+#include <QSysInfo>
 
+#include "BuiltinFeatures.h"
 #include "ComputerControlInterface.h"
 #include "Filesystem.h"
+#include "HostAddress.h"
 #include "Logger.h"
 #include "NetworkObjectDirectoryManager.h"
 #include "PasswordDialog.h"
@@ -62,11 +65,13 @@ VeyonCore::VeyonCore( QCoreApplication* application, const QString& appComponent
 	m_pluginManager( nullptr ),
 	m_platformPluginManager( nullptr ),
 	m_platformPlugin( nullptr ),
+	m_builtinFeatures( nullptr ),
 	m_userGroupsBackendManager( nullptr ),
 	m_networkObjectDirectoryManager( nullptr ),
 	m_localComputerControlInterface( nullptr ),
 	m_applicationName( QStringLiteral( "Veyon" ) ),
-	m_authenticationKeyName()
+	m_authenticationKeyName(),
+	m_debugging( false )
 {
 	Q_ASSERT( application != nullptr );
 
@@ -84,13 +89,15 @@ VeyonCore::VeyonCore( QCoreApplication* application, const QString& appComponent
 
 	initCryptoCore();
 
-	initAuthentication( AuthenticationCredentials::None );
+	initAuthenticationCredentials();
 
 	initPlugins();
 
 	initManagers();
 
 	initLocalComputerControlInterface();
+
+	initSystemInfo();
 }
 
 
@@ -109,6 +116,9 @@ VeyonCore::~VeyonCore()
 	delete m_logger;
 	m_logger = nullptr;
 
+	delete m_builtinFeatures;
+	m_builtinFeatures = nullptr;
+
 	delete m_platformPluginManager;
 	m_platformPluginManager = nullptr;
 
@@ -151,7 +161,7 @@ QString VeyonCore::pluginDir()
 
 QString VeyonCore::translationsDirectory()
 {
-	return QCoreApplication::applicationDirPath() + QDir::separator() + VEYON_TRANSLATIONS_DIR;
+	return QCoreApplication::applicationDirPath() + QDir::separator() + QStringLiteral(VEYON_TRANSLATIONS_DIR);
 }
 
 
@@ -169,7 +179,7 @@ QString VeyonCore::qtTranslationsDirectory()
 
 QString VeyonCore::executableSuffix()
 {
-	return QStringLiteral( VEYON_EXECUTABLE_SUFFIX );
+	return QStringLiteral( VEYON_EXECUTABLE_SUFFIX ); // clazy:exclude=empty-qstringliteral
 }
 
 
@@ -199,50 +209,15 @@ void VeyonCore::setupApplicationParameters()
 
 
 
-bool VeyonCore::initAuthentication( int credentialTypes )
+bool VeyonCore::initAuthentication()
 {
-	if( m_authenticationCredentials )
+	switch( config().authenticationMethod() )
 	{
-		delete m_authenticationCredentials;
-		m_authenticationCredentials = nullptr;
+	case AuthenticationMethod::LogonAuthentication: return initLogonAuthentication();
+	case AuthenticationMethod::KeyFileAuthentication: return initKeyFileAuthentication();
 	}
 
-	m_authenticationCredentials = new AuthenticationCredentials;
-
-	bool success = false;
-
-	if( credentialTypes & AuthenticationCredentials::UserLogon &&
-			config().authenticationMethod() == LogonAuthentication )
-	{
-		if( qobject_cast<QApplication *>( QCoreApplication::instance() ) )
-		{
-			PasswordDialog dlg( QApplication::activeWindow() );
-			if( dlg.exec() &&
-				dlg.credentials().hasCredentials( AuthenticationCredentials::UserLogon ) )
-			{
-				m_authenticationCredentials->setLogonUsername( dlg.username() );
-				m_authenticationCredentials->setLogonPassword( dlg.password() );
-
-				success = true;
-			}
-			else
-			{
-				success = false;
-			}
-		}
-		else
-		{
-			success = false;
-		}
-	}
-
-	if( credentialTypes & AuthenticationCredentials::PrivateKey &&
-			config().authenticationMethod() == KeyFileAuthentication )
-	{
-		success = initKeyFileAuthentication();
-	}
-
-	return success;
+	return false;
 }
 
 
@@ -309,7 +284,178 @@ void VeyonCore::enforceBranding( QWidget *topLevelWidget )
 
 bool VeyonCore::isDebugging()
 {
-	return instance()->m_logger->logLevel() >= Logger::LogLevelDebug;
+	return instance()->m_debugging;
+}
+
+
+
+QByteArray VeyonCore::shortenFuncinfo( QByteArray info )
+{
+	const auto funcinfo = cleanupFuncinfo( info );
+
+	if( isDebugging() )
+	{
+		return funcinfo + QByteArrayLiteral( "():" );
+	}
+
+	return funcinfo.split( ':' ).first() + QByteArrayLiteral(":");
+}
+
+
+
+// taken from qtbase/src/corelib/global/qlogging.cpp
+
+QByteArray VeyonCore::cleanupFuncinfo( QByteArray info )
+{
+	// Strip the function info down to the base function name
+	// note that this throws away the template definitions,
+	// the parameter types (overloads) and any const/volatile qualifiers.
+
+	if (info.isEmpty())
+		return info;
+
+	int pos;
+
+	// Skip trailing [with XXX] for templates (gcc), but make
+	// sure to not affect Objective-C message names.
+	pos = info.size() - 1;
+	if (info.endsWith(']') && !(info.startsWith('+') || info.startsWith('-'))) {
+		while (--pos) {
+			if (info.at(pos) == '[')
+				info.truncate(pos);
+		}
+	}
+
+	// operator names with '(', ')', '<', '>' in it
+	static const char operator_call[] = "operator()";
+	static const char operator_lessThan[] = "operator<";
+	static const char operator_greaterThan[] = "operator>";
+	static const char operator_lessThanEqual[] = "operator<=";
+	static const char operator_greaterThanEqual[] = "operator>=";
+
+	// canonize operator names
+	info.replace("operator ", "operator");
+
+	// remove argument list
+	forever {
+		int parencount = 0;
+		pos = info.lastIndexOf(')');
+		if (pos == -1) {
+			// Don't know how to parse this function name
+			return info;
+		}
+
+		// find the beginning of the argument list
+		--pos;
+		++parencount;
+		while (pos && parencount) {
+			if (info.at(pos) == ')')
+				++parencount;
+			else if (info.at(pos) == '(')
+				--parencount;
+			--pos;
+		}
+		if (parencount != 0)
+			return info;
+
+		info.truncate(++pos);
+
+		if (info.at(pos - 1) == ')') {
+			if (info.indexOf(operator_call) == pos - (int)strlen(operator_call))
+				break;
+
+			// this function returns a pointer to a function
+			// and we matched the arguments of the return type's parameter list
+			// try again
+			info.remove(0, info.indexOf('('));
+			info.chop(1);
+			continue;
+		} else {
+			break;
+		}
+	}
+
+	// find the beginning of the function name
+	int parencount = 0;
+	int templatecount = 0;
+	--pos;
+
+	// make sure special characters in operator names are kept
+	if (pos > -1) {
+		switch (info.at(pos)) {
+		case ')':
+			if (info.indexOf(operator_call) == pos - (int)strlen(operator_call) + 1)
+				pos -= 2;
+			break;
+		case '<':
+			if (info.indexOf(operator_lessThan) == pos - (int)strlen(operator_lessThan) + 1)
+				--pos;
+			break;
+		case '>':
+			if (info.indexOf(operator_greaterThan) == pos - (int)strlen(operator_greaterThan) + 1)
+				--pos;
+			break;
+		case '=': {
+			int operatorLength = (int)strlen(operator_lessThanEqual);
+			if (info.indexOf(operator_lessThanEqual) == pos - operatorLength + 1)
+				pos -= 2;
+			else if (info.indexOf(operator_greaterThanEqual) == pos - operatorLength + 1)
+				pos -= 2;
+			break;
+		}
+		default:
+			break;
+		}
+	}
+
+	while (pos > -1) {
+		if (parencount < 0 || templatecount < 0)
+			return info;
+
+		char c = info.at(pos);
+		if (c == ')')
+			++parencount;
+		else if (c == '(')
+			--parencount;
+		else if (c == '>')
+			++templatecount;
+		else if (c == '<')
+			--templatecount;
+		else if (c == ' ' && templatecount == 0 && parencount == 0)
+			break;
+
+		--pos;
+	}
+	info = info.mid(pos + 1);
+
+	// remove trailing '*', '&' that are part of the return argument
+	while ((info.at(0) == '*')
+		   || (info.at(0) == '&'))
+		info = info.mid(1);
+
+	// we have the full function name now.
+	// clean up the templates
+	while ((pos = info.lastIndexOf('>')) != -1) {
+		if (!info.contains('<'))
+			break;
+
+		// find the matching close
+		int end = pos;
+		templatecount = 1;
+		--pos;
+		while (pos && templatecount) {
+			char c = info.at(pos);
+			if (c == '>')
+				++templatecount;
+			else if (c == '<')
+				--templatecount;
+			--pos;
+		}
+		++pos;
+		info.remove(pos, end - pos + 1);
+	}
+
+	return info;
 }
 
 
@@ -317,7 +463,7 @@ bool VeyonCore::isDebugging()
 QString VeyonCore::stripDomain( const QString& username )
 {
 	// remove the domain part of username (e.g. "EXAMPLE.COM\Teacher" -> "Teacher")
-	int domainSeparator = username.indexOf( '\\' );
+	int domainSeparator = username.indexOf( QLatin1Char('\\') );
 	if( domainSeparator >= 0 )
 	{
 		return username.mid( domainSeparator + 1 );
@@ -341,7 +487,22 @@ QString VeyonCore::formattedUuid( QUuid uuid )
 
 bool VeyonCore::isAuthenticationKeyNameValid( const QString& authKeyName )
 {
-	return QRegExp( "\\w+" ).exactMatch( authKeyName );
+	return QRegExp( QStringLiteral("\\w+") ).exactMatch( authKeyName );
+}
+
+
+
+int VeyonCore::exec()
+{
+	emit applicationLoaded();
+
+	vDebug() << "Running";
+
+	const auto result = QCoreApplication::instance()->exec();
+
+	vDebug() << "Exit";
+
+	return result;
 }
 
 
@@ -362,6 +523,7 @@ void VeyonCore::initPlatformPlugin()
 void VeyonCore::initConfiguration()
 {
 	m_config = new VeyonConfiguration();
+	m_config->upgrade();
 
 	if( QUuid( config().installationID() ).isNull() )
 	{
@@ -387,6 +549,8 @@ void VeyonCore::initLogging( const QString& appComponentName )
 		m_logger = new Logger( appComponentName );
 	}
 
+	m_debugging = ( m_logger->logLevel() >= Logger::LogLevel::Debug );
+
 	VncConnection::initLogging( isDebugging() );
 }
 
@@ -404,13 +568,16 @@ void VeyonCore::initLocaleAndTranslation()
 
 	if( configuredLocale.language() != QLocale::English )
 	{
-
 		auto tr = new QTranslator;
 		if( configuredLocale == QLocale::C ||
-				tr->load( QStringLiteral( "%1.qm" ).arg( configuredLocale.name() ), translationsDirectory() ) == false )
+			tr->load( QStringLiteral( "%1.qm" ).arg( configuredLocale.name() ), translationsDirectory() ) == false )
 		{
 			configuredLocale = QLocale::system(); // Flawfinder: ignore
-			tr->load( QStringLiteral( "%1.qm" ).arg( QLocale::system().name() ), translationsDirectory() ); // Flawfinder: ignore
+
+			if( tr->load( QStringLiteral( "%1.qm" ).arg( configuredLocale.name() ), translationsDirectory() ) == false )
+			{
+				tr->load( QStringLiteral( "%1.qm" ).arg( configuredLocale.language() ), translationsDirectory() );
+			}
 		}
 
 		QLocale::setDefault( configuredLocale );
@@ -418,7 +585,10 @@ void VeyonCore::initLocaleAndTranslation()
 		QCoreApplication::installTranslator( tr );
 
 		auto qtTr = new QTranslator;
-		qtTr->load( QStringLiteral( "qt_%1.qm" ).arg( configuredLocale.name() ), qtTranslationsDirectory() );
+		if( qtTr->load( QStringLiteral( "qt_%1.qm" ).arg( configuredLocale.name() ), qtTranslationsDirectory() ) == false )
+		{
+			qtTr->load( QStringLiteral( "qt_%1.qm" ).arg( configuredLocale.language() ), qtTranslationsDirectory() );
+		}
 
 		QCoreApplication::installTranslator( qtTr );
 	}
@@ -426,7 +596,11 @@ void VeyonCore::initLocaleAndTranslation()
 	if( configuredLocale.language() == QLocale::Hebrew ||
 		configuredLocale.language() == QLocale::Arabic )
 	{
-		QApplication::setLayoutDirection( Qt::RightToLeft );
+		auto app = qobject_cast<QApplication *>( QCoreApplication::instance() );
+		if( app )
+		{
+			QApplication::setLayoutDirection( Qt::RightToLeft );
+		}
 	}
 }
 
@@ -439,11 +613,26 @@ void VeyonCore::initCryptoCore()
 
 
 
+void VeyonCore::initAuthenticationCredentials()
+{
+	if( m_authenticationCredentials )
+	{
+		delete m_authenticationCredentials;
+		m_authenticationCredentials = nullptr;
+	}
+
+	m_authenticationCredentials = new AuthenticationCredentials;
+}
+
+
+
 void VeyonCore::initPlugins()
 {
 	// load all other (non-platform) plugins
 	m_pluginManager->loadPlugins();
 	m_pluginManager->upgradePlugins();
+
+	m_builtinFeatures = new BuiltinFeatures();
 }
 
 
@@ -467,6 +656,26 @@ void VeyonCore::initLocalComputerControlInterface()
 
 
 
+bool VeyonCore::initLogonAuthentication()
+{
+	if( qobject_cast<QApplication *>( QCoreApplication::instance() ) )
+	{
+		PasswordDialog dlg( QApplication::activeWindow() );
+		if( dlg.exec() &&
+			dlg.credentials().hasCredentials( AuthenticationCredentials::Type::UserLogon ) )
+		{
+			m_authenticationCredentials->setLogonUsername( dlg.username() );
+			m_authenticationCredentials->setLogonPassword( dlg.password() );
+
+			return true;
+		}
+	}
+
+	return false;
+}
+
+
+
 bool VeyonCore::initKeyFileAuthentication()
 {
 	auto authKeyName = QProcessEnvironment::systemEnvironment().value( QStringLiteral("VEYON_AUTH_KEY_NAME") );
@@ -497,3 +706,12 @@ bool VeyonCore::initKeyFileAuthentication()
 
 	return false;
 }
+
+
+
+void VeyonCore::initSystemInfo()
+{
+	vDebug() << version() << HostAddress::localFQDN()
+			 << QSysInfo::kernelType() << QSysInfo::kernelVersion()
+			 << QSysInfo::prettyProductName() << QSysInfo::productType() << QSysInfo::productVersion();
+}
