@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2006-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -24,11 +24,13 @@
 
 #define XK_KOREAN
 #include "rfb/keysym.h"
+#include "rfb/rfbproto.h"
 
 #include "VncView.h"
 #include "PlatformInputDeviceFunctions.h"
 #include "KeyboardShortcutTrapper.h"
 #include "ProgressWidget.h"
+#include "VeyonConnection.h"
 
 #include <QApplication>
 #include <QDesktopWidget>
@@ -40,6 +42,7 @@
 VncView::VncView( const QString &host, int port, QWidget *parent, Mode mode ) :
 	QWidget( parent ),
 	m_vncConn( new VncConnection( QCoreApplication::instance() ) ),
+	m_veyonConnection( new VeyonConnection( m_vncConn ) ),
 	m_mode( mode ),
 	m_cursorShape(),
 	m_cursorX( 0 ),
@@ -60,17 +63,17 @@ VncView::VncView( const QString &host, int port, QWidget *parent, Mode mode ) :
 
 	if( m_mode == DemoMode )
 	{
-		m_vncConn->setQuality( VncConnection::DefaultQuality );
-		m_vncConn->setVeyonAuthType( RfbVeyonAuth::HostWhiteList );
+		m_vncConn->setQuality( VncConnection::Quality::Default );
+		m_veyonConnection->setVeyonAuthType( RfbVeyonAuth::Token );
 		m_establishingConnectionWidget = new ProgressWidget(
-			tr( "Establishing connection to %1 ..." ).arg( m_vncConn->host() ),
-					QStringLiteral( ":/resources/watch%1.png" ), 16, this );
+					tr( "Establishing connection to %1 ..." ).arg( m_vncConn->host() ),
+					QStringLiteral( ":/core/watch%1.png" ), 16, this );
 		connect( m_vncConn, &VncConnection::stateChanged,
 				 this, &VncView::updateConnectionState );
 	}
 	else if( m_mode == RemoteControlMode )
 	{
-		m_vncConn->setQuality( VncConnection::RemoteControlQuality );
+		m_vncConn->setQuality( VncConnection::Quality::RemoteControl );
 	}
 
 	connect( m_vncConn, &VncConnection::imageUpdated, this, &VncView::updateImage );
@@ -117,6 +120,9 @@ VncView::~VncView()
 	unpressModifiers();
 	delete m_keyboardShortcutTrapper;
 
+	delete m_veyonConnection;
+	m_veyonConnection = nullptr;
+
 	m_vncConn->stopAndDeleteLater();
 	m_vncConn = nullptr;
 }
@@ -133,7 +139,9 @@ bool VncView::eventFilter(QObject *obj, QEvent *event)
 			event->type() == QEvent::MouseButtonPress ||
 			event->type() == QEvent::MouseButtonRelease ||
 			event->type() == QEvent::Wheel )
-		return true;
+		{
+			return true;
+		}
 	}
 
 	return QWidget::eventFilter(obj, event);
@@ -245,7 +253,7 @@ void VncView::sendShortcut( VncView::Shortcut shortcut )
 		unpressKey( XK_Control_L );
 		break;
 	default:
-		qWarning( "VncView::sendShortcut(): unknown shortcut %d", static_cast<int>( shortcut ) );
+		vWarning() << "unknown shortcut" << static_cast<int>( shortcut );
 		break;
 	}
 }
@@ -349,6 +357,11 @@ void VncView::focusOutEvent( QFocusEvent* event )
 // our builtin keyboard-handler
 void VncView::keyEventHandler( QKeyEvent* event )
 {
+	if( event == nullptr )
+	{
+		return;
+	}
+
 	bool pressed = event->type() == QEvent::KeyPress;
 
 #ifdef Q_OS_LINUX
@@ -368,87 +381,87 @@ void VncView::keyEventHandler( QKeyEvent* event )
 	unsigned int key = 0;
 	switch( event->key() )
 	{
-		// modifiers are handled separately
-		case Qt::Key_Shift: key = XK_Shift_L; break;
-		case Qt::Key_Control: key = XK_Control_L; break;
-		case Qt::Key_Meta: key = XK_Meta_L; break;
-		case Qt::Key_Alt: key = XK_Alt_L; break;
-		case Qt::Key_Escape: key = XK_Escape; break;
-		case Qt::Key_Tab: key = XK_Tab; break;
-		case Qt::Key_Backtab: key = XK_Tab; break;
-		case Qt::Key_Backspace: key = XK_BackSpace; break;
-		case Qt::Key_Return: key = XK_Return; break;
-		case Qt::Key_Insert: key = XK_Insert; break;
-		case Qt::Key_Delete: key = XK_Delete; break;
-		case Qt::Key_Pause: key = XK_Pause; break;
-		case Qt::Key_Print: key = XK_Print; break;
-		case Qt::Key_Home: key = XK_Home; break;
-		case Qt::Key_End: key = XK_End; break;
-		case Qt::Key_Left: key = XK_Left; break;
-		case Qt::Key_Up: key = XK_Up; break;
-		case Qt::Key_Right: key = XK_Right; break;
-		case Qt::Key_Down: key = XK_Down; break;
-		case Qt::Key_PageUp: key = XK_Prior; break;
-		case Qt::Key_PageDown: key = XK_Next; break;
-		case Qt::Key_CapsLock: key = XK_Caps_Lock; break;
-		case Qt::Key_NumLock: key = XK_Num_Lock; break;
-		case Qt::Key_ScrollLock: key = XK_Scroll_Lock; break;
-		case Qt::Key_Super_L: key = XK_Super_L; break;
-		case Qt::Key_Super_R: key = XK_Super_R; break;
-		case Qt::Key_Menu: key = XK_Menu; break;
-		case Qt::Key_Hyper_L: key = XK_Hyper_L; break;
-		case Qt::Key_Hyper_R: key = XK_Hyper_R; break;
-		case Qt::Key_Help: key = XK_Help; break;
-		case Qt::Key_AltGr: key = XK_ISO_Level3_Shift; break;
-		case Qt::Key_Multi_key: key = XK_Multi_key; break;
-		case Qt::Key_SingleCandidate: key = XK_SingleCandidate; break;
-		case Qt::Key_MultipleCandidate: key = XK_MultipleCandidate; break;
-		case Qt::Key_PreviousCandidate: key = XK_PreviousCandidate; break;
-		case Qt::Key_Mode_switch: key = XK_Mode_switch; break;
-		case Qt::Key_Kanji: key = XK_Kanji; break;
-		case Qt::Key_Muhenkan: key = XK_Muhenkan; break;
-		case Qt::Key_Henkan: key = XK_Henkan; break;
-		case Qt::Key_Romaji: key = XK_Romaji; break;
-		case Qt::Key_Hiragana: key = XK_Hiragana; break;
-		case Qt::Key_Katakana: key = XK_Katakana; break;
-		case Qt::Key_Hiragana_Katakana: key = XK_Hiragana_Katakana; break;
-		case Qt::Key_Zenkaku: key = XK_Zenkaku; break;
-		case Qt::Key_Hankaku: key = XK_Hankaku; break;
-		case Qt::Key_Zenkaku_Hankaku: key = XK_Zenkaku_Hankaku; break;
-		case Qt::Key_Touroku: key = XK_Touroku; break;
-		case Qt::Key_Massyo: key = XK_Massyo; break;
-		case Qt::Key_Kana_Lock: key = XK_Kana_Lock; break;
-		case Qt::Key_Kana_Shift: key = XK_Kana_Shift; break;
-		case Qt::Key_Eisu_Shift: key = XK_Eisu_Shift; break;
-		case Qt::Key_Eisu_toggle: key = XK_Eisu_toggle; break;
-		case Qt::Key_Hangul: key = XK_Hangul; break;
-		case Qt::Key_Hangul_Start: key = XK_Hangul_Start; break;
-		case Qt::Key_Hangul_End: key = XK_Hangul_End; break;
-		case Qt::Key_Hangul_Hanja: key = XK_Hangul_Hanja; break;
-		case Qt::Key_Hangul_Jamo: key = XK_Hangul_Jamo; break;
-		case Qt::Key_Hangul_Romaja: key = XK_Hangul_Romaja; break;
-		case Qt::Key_Hangul_Jeonja: key = XK_Hangul_Jeonja; break;
-		case Qt::Key_Hangul_Banja: key = XK_Hangul_Banja; break;
-		case Qt::Key_Hangul_PreHanja: key = XK_Hangul_PreHanja; break;
-		case Qt::Key_Hangul_PostHanja: key = XK_Hangul_PostHanja; break;
-		case Qt::Key_Hangul_Special: key = XK_Hangul_Special; break;
-		case Qt::Key_Dead_Grave: key = XK_dead_grave; break;
-		case Qt::Key_Dead_Acute: key = XK_dead_acute; break;
-		case Qt::Key_Dead_Circumflex: key = XK_dead_circumflex; break;
-		case Qt::Key_Dead_Tilde: key = XK_dead_tilde; break;
-		case Qt::Key_Dead_Macron: key = XK_dead_macron; break;
-		case Qt::Key_Dead_Breve: key = XK_dead_breve; break;
-		case Qt::Key_Dead_Abovedot: key = XK_dead_abovedot; break;
-		case Qt::Key_Dead_Diaeresis: key = XK_dead_diaeresis; break;
-		case Qt::Key_Dead_Abovering: key = XK_dead_abovering; break;
-		case Qt::Key_Dead_Doubleacute: key = XK_dead_doubleacute; break;
-		case Qt::Key_Dead_Caron: key = XK_dead_caron; break;
-		case Qt::Key_Dead_Cedilla: key = XK_dead_cedilla; break;
-		case Qt::Key_Dead_Ogonek: key = XK_dead_ogonek; break;
-		case Qt::Key_Dead_Iota: key = XK_dead_iota; break;
-		case Qt::Key_Dead_Voiced_Sound: key = XK_dead_voiced_sound; break;
-		case Qt::Key_Dead_Semivoiced_Sound: key = XK_dead_semivoiced_sound; break;
-		case Qt::Key_Dead_Belowdot: key = XK_dead_belowdot; break;
+	// modifiers are handled separately
+	case Qt::Key_Shift: key = XK_Shift_L; break;
+	case Qt::Key_Control: key = XK_Control_L; break;
+	case Qt::Key_Meta: key = XK_Meta_L; break;
+	case Qt::Key_Alt: key = XK_Alt_L; break;
+	case Qt::Key_Escape: key = XK_Escape; break;
+	case Qt::Key_Tab: key = XK_Tab; break;
+	case Qt::Key_Backtab: key = XK_Tab; break;
+	case Qt::Key_Backspace: key = XK_BackSpace; break;
+	case Qt::Key_Return: key = XK_Return; break;
+	case Qt::Key_Insert: key = XK_Insert; break;
+	case Qt::Key_Delete: key = XK_Delete; break;
+	case Qt::Key_Pause: key = XK_Pause; break;
+	case Qt::Key_Print: key = XK_Print; break;
+	case Qt::Key_Home: key = XK_Home; break;
+	case Qt::Key_End: key = XK_End; break;
+	case Qt::Key_Left: key = XK_Left; break;
+	case Qt::Key_Up: key = XK_Up; break;
+	case Qt::Key_Right: key = XK_Right; break;
+	case Qt::Key_Down: key = XK_Down; break;
+	case Qt::Key_PageUp: key = XK_Prior; break;
+	case Qt::Key_PageDown: key = XK_Next; break;
+	case Qt::Key_CapsLock: key = XK_Caps_Lock; break;
+	case Qt::Key_NumLock: key = XK_Num_Lock; break;
+	case Qt::Key_ScrollLock: key = XK_Scroll_Lock; break;
+	case Qt::Key_Super_L: key = XK_Super_L; break;
+	case Qt::Key_Super_R: key = XK_Super_R; break;
+	case Qt::Key_Menu: key = XK_Menu; break;
+	case Qt::Key_Hyper_L: key = XK_Hyper_L; break;
+	case Qt::Key_Hyper_R: key = XK_Hyper_R; break;
+	case Qt::Key_Help: key = XK_Help; break;
+	case Qt::Key_AltGr: key = XK_ISO_Level3_Shift; break;
+	case Qt::Key_Multi_key: key = XK_Multi_key; break;
+	case Qt::Key_SingleCandidate: key = XK_SingleCandidate; break;
+	case Qt::Key_MultipleCandidate: key = XK_MultipleCandidate; break;
+	case Qt::Key_PreviousCandidate: key = XK_PreviousCandidate; break;
+	case Qt::Key_Mode_switch: key = XK_Mode_switch; break;
+	case Qt::Key_Kanji: key = XK_Kanji; break;
+	case Qt::Key_Muhenkan: key = XK_Muhenkan; break;
+	case Qt::Key_Henkan: key = XK_Henkan; break;
+	case Qt::Key_Romaji: key = XK_Romaji; break;
+	case Qt::Key_Hiragana: key = XK_Hiragana; break;
+	case Qt::Key_Katakana: key = XK_Katakana; break;
+	case Qt::Key_Hiragana_Katakana: key = XK_Hiragana_Katakana; break;
+	case Qt::Key_Zenkaku: key = XK_Zenkaku; break;
+	case Qt::Key_Hankaku: key = XK_Hankaku; break;
+	case Qt::Key_Zenkaku_Hankaku: key = XK_Zenkaku_Hankaku; break;
+	case Qt::Key_Touroku: key = XK_Touroku; break;
+	case Qt::Key_Massyo: key = XK_Massyo; break;
+	case Qt::Key_Kana_Lock: key = XK_Kana_Lock; break;
+	case Qt::Key_Kana_Shift: key = XK_Kana_Shift; break;
+	case Qt::Key_Eisu_Shift: key = XK_Eisu_Shift; break;
+	case Qt::Key_Eisu_toggle: key = XK_Eisu_toggle; break;
+	case Qt::Key_Hangul: key = XK_Hangul; break;
+	case Qt::Key_Hangul_Start: key = XK_Hangul_Start; break;
+	case Qt::Key_Hangul_End: key = XK_Hangul_End; break;
+	case Qt::Key_Hangul_Hanja: key = XK_Hangul_Hanja; break;
+	case Qt::Key_Hangul_Jamo: key = XK_Hangul_Jamo; break;
+	case Qt::Key_Hangul_Romaja: key = XK_Hangul_Romaja; break;
+	case Qt::Key_Hangul_Jeonja: key = XK_Hangul_Jeonja; break;
+	case Qt::Key_Hangul_Banja: key = XK_Hangul_Banja; break;
+	case Qt::Key_Hangul_PreHanja: key = XK_Hangul_PreHanja; break;
+	case Qt::Key_Hangul_PostHanja: key = XK_Hangul_PostHanja; break;
+	case Qt::Key_Hangul_Special: key = XK_Hangul_Special; break;
+	case Qt::Key_Dead_Grave: key = XK_dead_grave; break;
+	case Qt::Key_Dead_Acute: key = XK_dead_acute; break;
+	case Qt::Key_Dead_Circumflex: key = XK_dead_circumflex; break;
+	case Qt::Key_Dead_Tilde: key = XK_dead_tilde; break;
+	case Qt::Key_Dead_Macron: key = XK_dead_macron; break;
+	case Qt::Key_Dead_Breve: key = XK_dead_breve; break;
+	case Qt::Key_Dead_Abovedot: key = XK_dead_abovedot; break;
+	case Qt::Key_Dead_Diaeresis: key = XK_dead_diaeresis; break;
+	case Qt::Key_Dead_Abovering: key = XK_dead_abovering; break;
+	case Qt::Key_Dead_Doubleacute: key = XK_dead_doubleacute; break;
+	case Qt::Key_Dead_Caron: key = XK_dead_caron; break;
+	case Qt::Key_Dead_Cedilla: key = XK_dead_cedilla; break;
+	case Qt::Key_Dead_Ogonek: key = XK_dead_ogonek; break;
+	case Qt::Key_Dead_Iota: key = XK_dead_iota; break;
+	case Qt::Key_Dead_Voiced_Sound: key = XK_dead_voiced_sound; break;
+	case Qt::Key_Dead_Semivoiced_Sound: key = XK_dead_semivoiced_sound; break;
+	case Qt::Key_Dead_Belowdot: key = XK_dead_belowdot; break;
 	}
 
 	if( event->key() >= Qt::Key_F1 && event->key() <= Qt::Key_F35 )
@@ -475,7 +488,7 @@ void VncView::keyEventHandler( QKeyEvent* event )
 	// correct translation of AltGr+<character key> (non-US-keyboard layout
 	// such as German keyboard layout)
 	if( m_mods.contains( XK_Alt_L ) && m_mods.contains( XK_Control_L ) &&
-						key >= 64 && key < 0xF000 )
+		key >= 64 && key < 0xF000 )
 	{
 		unpressModifiers();
 		m_vncConn->keyEvent( XK_ISO_Level3_Shift, true );
@@ -484,9 +497,9 @@ void VncView::keyEventHandler( QKeyEvent* event )
 
 	// handle Ctrl+Alt+Del replacement (Meta/Super key+Del)
 	if( ( m_mods.contains( XK_Super_L ) ||
-			m_mods.contains( XK_Super_R ) ||
-			m_mods.contains( XK_Meta_L ) ) &&
-				event->key() == Qt::Key_Delete )
+		  m_mods.contains( XK_Super_R ) ||
+		  m_mods.contains( XK_Meta_L ) ) &&
+		event->key() == Qt::Key_Delete )
 	{
 		if( pressed )
 		{
@@ -503,7 +516,7 @@ void VncView::keyEventHandler( QKeyEvent* event )
 
 	// handle modifiers
 	if( key == XK_Shift_L || key == XK_Control_L || key == XK_Meta_L ||
-			key == XK_Alt_L || key == XK_Super_L || key == XK_Super_R )
+		key == XK_Alt_L || key == XK_Super_L || key == XK_Super_R )
 	{
 		if( pressed )
 		{
@@ -572,11 +585,11 @@ QPoint VncView::mapToFramebuffer( QPoint pos )
 {
 	if( m_framebufferSize.isEmpty() )
 	{
-		return QPoint( 0, 0 );
+		return { 0, 0 };
 	}
 
-	return QPoint( pos.x() * m_framebufferSize.width() / scaledSize().width(),
-				   pos.y() * m_framebufferSize.height() / scaledSize().height() );
+	return { pos.x() * m_framebufferSize.width() / scaledSize().width(),
+				pos.y() * m_framebufferSize.height() / scaledSize().height() };
 }
 
 
@@ -585,14 +598,14 @@ QRect VncView::mapFromFramebuffer( QRect r )
 {
 	if( m_framebufferSize.isEmpty() )
 	{
-		return QRect();
+		return {};
 	}
 
 	const auto dx = scaledSize().width() / static_cast<qreal>( m_framebufferSize.width() );
 	const auto dy = scaledSize().height() / static_cast<qreal>( m_framebufferSize.height() );
 
-	return( QRect( static_cast<int>(r.x()*dx), static_cast<int>(r.y()*dy),
-				   static_cast<int>(r.width()*dx), static_cast<int>(r.height()*dy) ) );
+	return { static_cast<int>(r.x()*dx), static_cast<int>(r.y()*dy),
+				static_cast<int>(r.width()*dx), static_cast<int>(r.height()*dy) };
 }
 
 
@@ -633,21 +646,21 @@ bool VncView::event( QEvent * event )
 {
 	switch( event->type() )
 	{
-		case QEvent::KeyPress:
-		case QEvent::KeyRelease:
-			keyEventHandler( static_cast<QKeyEvent*>( event ) );
-			return true;
-		case QEvent::MouseButtonDblClick:
-		case QEvent::MouseButtonPress:
-		case QEvent::MouseButtonRelease:
-		case QEvent::MouseMove:
-			mouseEventHandler( static_cast<QMouseEvent*>( event ) );
-			return true;
-		case QEvent::Wheel:
-			wheelEventHandler( static_cast<QWheelEvent*>( event ) );
-			return true;
-		default:
-			return QWidget::event(event);
+	case QEvent::KeyPress:
+	case QEvent::KeyRelease:
+		keyEventHandler( dynamic_cast<QKeyEvent*>( event ) );
+		return true;
+	case QEvent::MouseButtonDblClick:
+	case QEvent::MouseButtonPress:
+	case QEvent::MouseButtonRelease:
+	case QEvent::MouseMove:
+		mouseEventHandler( dynamic_cast<QMouseEvent*>( event ) );
+		return true;
+	case QEvent::Wheel:
+		wheelEventHandler( dynamic_cast<QWheelEvent*>( event ) );
+		return true;
+	default:
+		return QWidget::event(event);
 	}
 }
 
@@ -679,9 +692,9 @@ void VncView::paintEvent( QPaintEvent* paintEvent )
 	if( isViewOnly() && !m_cursorShape.isNull() )
 	{
 		const QRect cursorRect = mapFromFramebuffer(
-			QRect( QPoint( m_cursorX - m_cursorHotX,
-							m_cursorY - m_cursorHotY ),
-					m_cursorShape.size() ) );
+					QRect( QPoint( m_cursorX - m_cursorHotX,
+								   m_cursorY - m_cursorHotY ),
+						   m_cursorShape.size() ) );
 		// parts of cursor within updated region?
 		if( paintEvent->region().intersects( cursorRect ) )
 		{
@@ -724,7 +737,12 @@ void VncView::resizeEvent( QResizeEvent* event )
 
 void VncView::wheelEventHandler( QWheelEvent* event )
 {
-	const QPoint p = mapToFramebuffer( event->pos() );
+	if( event == nullptr )
+	{
+		return;
+	}
+
+	const auto p = mapToFramebuffer( event->pos() );
 	m_vncConn->mouseEvent( p.x(), p.y(), m_buttonMask | ( ( event->delta() < 0 ) ? rfbButton5Mask : rfbButton4Mask ) );
 	m_vncConn->mouseEvent( p.x(), p.y(), m_buttonMask );
 }
@@ -733,25 +751,31 @@ void VncView::wheelEventHandler( QWheelEvent* event )
 
 void VncView::mouseEventHandler( QMouseEvent* event )
 {
-	struct buttonXlate
+	if( event == nullptr )
+	{
+		return;
+	}
+
+	struct ButtonTranslation
 	{
 		Qt::MouseButton qt;
 		int rfb;
-	} const map[] =
-		{
-			{ Qt::LeftButton, rfbButton1Mask },
-			{ Qt::MidButton, rfbButton2Mask },
-			{ Qt::RightButton, rfbButton3Mask }
-		} ;
+	};
+
+	const ButtonTranslation buttonTranslationMap[] = {
+		{ Qt::LeftButton, rfbButton1Mask },
+		{ Qt::MidButton, rfbButton2Mask },
+		{ Qt::RightButton, rfbButton3Mask }
+	} ;
 
 	if( event->type() != QEvent::MouseMove )
 	{
-		for( auto i : map )
+		for( const auto& i : buttonTranslationMap )
 		{
 			if( event->button() == i.qt )
 			{
 				if( event->type() == QEvent::MouseButtonPress ||
-				event->type() == QEvent::MouseButtonDblClick )
+					event->type() == QEvent::MouseButtonDblClick )
 				{
 					m_buttonMask |= i.rfb;
 				}
@@ -826,6 +850,6 @@ void VncView::updateConnectionState()
 {
 	if( m_establishingConnectionWidget )
 	{
-		m_establishingConnectionWidget->setVisible( m_vncConn->state() != VncConnection::Connected );
+		m_establishingConnectionWidget->setVisible( m_vncConn->state() != VncConnection::State::Connected );
 	}
 }
