@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2017-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -23,7 +23,6 @@
  */
 
 #include <QPainter>
-#include <QTimer>
 
 #include "ComputerControlListModel.h"
 #include "ComputerManager.h"
@@ -32,15 +31,24 @@
 #include "UserConfig.h"
 #include "VeyonConfiguration.h"
 
+#if defined(QT_TESTLIB_LIB) && QT_VERSION >= QT_VERSION_CHECK(5, 11, 0)
+#include <QAbstractItemModelTester>
+#endif
+
 
 ComputerControlListModel::ComputerControlListModel( VeyonMaster* masterCore, QObject* parent ) :
 	QAbstractListModel( parent ),
 	m_master( masterCore ),
 	m_displayRoleContent( static_cast<DisplayRoleContent>( VeyonCore::config().computerDisplayRoleContent() ) ),
+	m_sortOrder( static_cast<SortOrder>( VeyonCore::config().computerDisplayRoleContent() ) ),
 	m_iconDefault(),
 	m_iconConnectionProblem(),
 	m_iconDemoMode()
 {
+#if defined(QT_TESTLIB_LIB) && QT_VERSION >= QT_VERSION_CHECK(5, 11, 0)
+	new QAbstractItemModelTester( this, QAbstractItemModelTester::FailureReportingMode::Warning, this );
+#endif
+
 	loadIcons();
 
 	connect( &m_master->computerManager(), &ComputerManager::computerSelectionReset,
@@ -48,10 +56,6 @@ ComputerControlListModel::ComputerControlListModel( VeyonMaster* masterCore, QOb
 	connect( &m_master->computerManager(), &ComputerManager::computerSelectionChanged,
 			 this, &ComputerControlListModel::update );
 
-	auto computerScreenUpdateTimer = new QTimer( this );
-	connect( computerScreenUpdateTimer, &QTimer::timeout, this, &ComputerControlListModel::updateComputerScreens );
-	computerScreenUpdateTimer->start( VeyonCore::config().computerMonitoringUpdateInterval() );
-
 	reload();
 }
 
@@ -78,7 +82,7 @@ QVariant ComputerControlListModel::data( const QModelIndex& index, int role ) co
 
 	if( index.row() >= m_computerControlInterfaces.count() )
 	{
-		qCritical( "ComputerListModel::data(): index out of range!" );
+		vCritical() << "index out of range!";
 	}
 
 	const auto computerControl = m_computerControlInterfaces[index.row()];
@@ -95,14 +99,13 @@ QVariant ComputerControlListModel::data( const QModelIndex& index, int role ) co
 		return computerDisplayRole( computerControl );
 
 	case Qt::InitialSortOrderRole:
-		return computerControl->computer().room() + computerControl->computer().name() +
-				computerControl->computer().hostAddress() + computerControl->user();
+		return computerSortRole( computerControl );
 
 	case UidRole:
 		return computerControl->computer().networkObjectUid();
 
 	case StateRole:
-		return computerControl->state();
+		return QVariant::fromValue( computerControl->state() );
 
 	default:
 		break;
@@ -129,7 +132,7 @@ ComputerControlInterface::Pointer ComputerControlListModel::computerControlInter
 {
 	if( index.isValid() == false || index.row() >= m_computerControlInterfaces.count() )
 	{
-		qCritical( "ComputerListModel::computerControlInterface(): invalid ComputerControlInterface requested!" );
+		vCritical() << "invalid ComputerControlInterface requested!";
 		return ComputerControlInterface::Pointer();
 	}
 
@@ -212,22 +215,36 @@ void ComputerControlListModel::update()
 
 
 
-void ComputerControlListModel::updateComputerScreens()
+void ComputerControlListModel::updateState( const QModelIndex& index )
 {
-	int computerIndex = 0;
+	emit dataChanged( index, index, { Qt::DisplayRole, Qt::DecorationRole, Qt::ToolTipRole } );
+}
 
-	for( auto& controlInterface : m_computerControlInterfaces )
+
+
+void ComputerControlListModel::updateScreen( const QModelIndex& index )
+{
+	emit dataChanged( index, index, { Qt::DecorationRole } );
+}
+
+
+
+void ComputerControlListModel::updateActiveFeatures( const QModelIndex& index )
+{
+	emit dataChanged( index, index, { Qt::ToolTipRole } );
+	emit activeFeaturesChanged( index );
+}
+
+
+
+void ComputerControlListModel::updateUser( const QModelIndex& index )
+{
+	emit dataChanged( index, index, { Qt::DisplayRole, Qt::ToolTipRole } );
+
+	auto controlInterface = computerControlInterface( index );
+	if( controlInterface.isNull() == false )
 	{
-		if( controlInterface->hasScreenUpdates() )
-		{
-			controlInterface->clearScreenUpdateFlag();
-
-			emit dataChanged( index( computerIndex, 0 ),
-							  index( computerIndex, 0 ),
-							  QVector<int>( { Qt::DisplayRole, Qt::DecorationRole, Qt::ToolTipRole } ) );
-		}
-
-		++computerIndex;
+		m_master->computerManager().updateUser( controlInterface );
 	}
 }
 
@@ -236,22 +253,24 @@ void ComputerControlListModel::updateComputerScreens()
 void ComputerControlListModel::startComputerControlInterface( const ComputerControlInterface::Pointer& controlInterface,
 															  const QModelIndex& index )
 {
-	controlInterface->start( computerScreenSize(), &m_master->builtinFeatures() );
+	controlInterface->start( computerScreenSize() );
 
 	connect( controlInterface.data(), &ComputerControlInterface::featureMessageReceived, this,
 			 [=]( const FeatureMessage& featureMessage, ComputerControlInterface::Pointer computerControlInterface ) {
 		m_master->featureManager().handleFeatureMessage( *m_master, featureMessage, computerControlInterface );
 	} );
 
+	connect( controlInterface.data(), &ComputerControlInterface::screenUpdated,
+			 this, [=] () { updateScreen( index ); } );
+
 	connect( controlInterface.data(), &ComputerControlInterface::activeFeaturesChanged,
-			 this, [=] () { emit activeFeaturesChanged( index ); } );
+			 this, [=] () { updateActiveFeatures( index ); } );
+
+	connect( controlInterface.data(), &ComputerControlInterface::stateChanged,
+			 this, [=] () { updateState( index ); } );
 
-	// pass weak pointer to lambda function as otherwise the original shared pointer
-	// gets referenced once more all the time and thus the object never gets deleted
-	auto controlInterfaceWeakRef = controlInterface->weakPointer();
 	connect( controlInterface.data(), &ComputerControlInterface::userChanged,
-			 &m_master->computerManager(),
-			 [=] () { m_master->computerManager().updateUser( controlInterfaceWeakRef ); } );
+			 this, [=]() { updateUser( index ); } );
 }
 
 
@@ -262,7 +281,8 @@ void ComputerControlListModel::stopComputerControlInterface( const ComputerContr
 
 	controlInterface->disconnect( &m_master->computerManager() );
 
-	controlInterface->setUser( QString() );
+	controlInterface->setUserLoginName( {} );
+	controlInterface->setUserFullName( {} );
 	m_master->computerManager().updateUser( controlInterface );
 }
 
@@ -270,17 +290,17 @@ void ComputerControlListModel::stopComputerControlInterface( const ComputerContr
 
 QSize ComputerControlListModel::computerScreenSize() const
 {
-	return QSize( m_master->userConfig().monitoringScreenSize(),
-				  m_master->userConfig().monitoringScreenSize() * 9 / 16 );
+	return { m_master->userConfig().monitoringScreenSize(),
+				m_master->userConfig().monitoringScreenSize() * 9 / 16 };
 }
 
 
 
 void ComputerControlListModel::loadIcons()
 {
-	m_iconDefault = prepareIcon( QImage( QStringLiteral(":/resources/preferences-desktop-display-gray.png") ) );
-	m_iconConnectionProblem = prepareIcon( QImage( QStringLiteral(":/resources/preferences-desktop-display-red.png") ) );
-	m_iconDemoMode = prepareIcon( QImage( QStringLiteral(":/resources/preferences-desktop-display-orange.png") ) );
+	m_iconDefault = prepareIcon( QImage( QStringLiteral(":/master/preferences-desktop-display-gray.png") ) );
+	m_iconConnectionProblem = prepareIcon( QImage( QStringLiteral(":/master/preferences-desktop-display-red.png") ) );
+	m_iconDemoMode = prepareIcon( QImage( QStringLiteral(":/master/preferences-desktop-display-orange.png") ) );
 }
 
 
@@ -302,7 +322,7 @@ QImage ComputerControlListModel::computerDecorationRole( const ComputerControlIn
 
 	switch( controlInterface->state() )
 	{
-	case ComputerControlInterface::Connected:
+	case ComputerControlInterface::State::Connected:
 		image = controlInterface->scaledScreen();
 		if( image.isNull() == false )
 		{
@@ -312,8 +332,8 @@ QImage ComputerControlListModel::computerDecorationRole( const ComputerControlIn
 		image = m_iconDefault;
 		break;
 
-	case ComputerControlInterface::AuthenticationFailed:
-	case ComputerControlInterface::ServiceUnreachable:
+	case ComputerControlInterface::State::AuthenticationFailed:
+	case ComputerControlInterface::State::ServiceUnreachable:
 		image = m_iconConnectionProblem;
 		break;
 
@@ -330,17 +350,17 @@ QImage ComputerControlListModel::computerDecorationRole( const ComputerControlIn
 QString ComputerControlListModel::computerToolTipRole( const ComputerControlInterface::Pointer& controlInterface ) const
 {
 	const QString state( computerStateDescription( controlInterface ) );
-	const QString room( tr( "Room: %1" ).arg( controlInterface->computer().room() ) );
+	const QString location( tr( "Location: %1" ).arg( controlInterface->computer().location() ) );
 	const QString host( tr( "Host/IP address: %1" ).arg( controlInterface->computer().hostAddress() ) );
 	const QString user( loggedOnUserInformation( controlInterface ) );
 	const QString features( tr( "Active features: %1" ).arg( activeFeatures( controlInterface ) ) );
 
 	if( user.isEmpty() )
 	{
-		return QStringLiteral( "<b>%1</b><br>%2<br>%3<br>%4" ).arg( state, room, host, features );
+		return QStringLiteral( "<b>%1</b><br>%2<br>%3<br>%4" ).arg( state, location, host, features );
 	}
 
-	return QStringLiteral( "<b>%1</b><br>%2<br>%3<br>%4<br>%5" ).arg( state, room, host, features, user);
+	return QStringLiteral( "<b>%1</b><br>%2<br>%3<br>%4<br>%5" ).arg( state, location, host, features, user);
 }
 
 
@@ -348,23 +368,13 @@ QString ComputerControlListModel::computerToolTipRole( const ComputerControlInte
 QString ComputerControlListModel::computerDisplayRole( const ComputerControlInterface::Pointer& controlInterface ) const
 {
 	if( m_displayRoleContent != DisplayComputerName &&
-			controlInterface->state() == ComputerControlInterface::Connected &&
-			controlInterface->user().isEmpty() == false )
+			controlInterface->state() == ComputerControlInterface::State::Connected &&
+			controlInterface->userLoginName().isEmpty() == false )
 	{
-		auto user = controlInterface->user();
-
-		// do we have full name information?
-		QRegExp fullNameRX( QStringLiteral("(.*) \\((.*)\\)") );
-		if( fullNameRX.indexIn( user ) >= 0 )
+		auto user = controlInterface->userFullName();
+		if( user.isEmpty() )
 		{
-			if( fullNameRX.cap( 2 ).isEmpty() == false )
-			{
-				user = fullNameRX.cap( 2 );
-			}
-			else
-			{
-				user = fullNameRX.cap( 1 );
-			}
+			user = controlInterface->userLoginName();
 		}
 
 		if( m_displayRoleContent == DisplayUserName )
@@ -382,7 +392,33 @@ QString ComputerControlListModel::computerDisplayRole( const ComputerControlInte
 		return controlInterface->computer().name();
 	}
 
-	return QString();
+	return {};
+}
+
+
+
+QString ComputerControlListModel::computerSortRole( const ComputerControlInterface::Pointer& controlInterface ) const
+{
+	switch( m_sortOrder )
+	{
+	case SortByComputerAndUserName:
+		return controlInterface->computer().location() + controlInterface->computer().name() +
+				controlInterface->computer().hostAddress() + controlInterface->userLoginName();
+
+	case SortByComputerName:
+		return controlInterface->computer().location() + controlInterface->computer().name() +
+				controlInterface->computer().hostAddress();
+
+	case SortByUserName:
+		if( controlInterface->userFullName().isEmpty() == false )
+		{
+			return controlInterface->userFullName();
+		}
+
+		return controlInterface->userLoginName();
+	}
+
+	return {};
 }
 
 
@@ -391,19 +427,19 @@ QString ComputerControlListModel::computerStateDescription( const ComputerContro
 {
 	switch( controlInterface->state() )
 	{
-	case ComputerControlInterface::Connected:
+	case ComputerControlInterface::State::Connected:
 		return tr( "Online and connected" );
 
-	case ComputerControlInterface::Connecting:
+	case ComputerControlInterface::State::Connecting:
 		return tr( "Establishing connection" );
 
-	case ComputerControlInterface::Offline:
+	case ComputerControlInterface::State::HostOffline:
 		return tr( "Computer offline or switched off" );
 
-	case ComputerControlInterface::ServiceUnreachable:
+	case ComputerControlInterface::State::ServiceUnreachable:
 		return tr( "Service unreachable or not running" );
 
-	case ComputerControlInterface::AuthenticationFailed:
+	case ComputerControlInterface::State::AuthenticationFailed:
 		return tr( "Authentication failed or access denied" );
 
 	default:
@@ -417,17 +453,23 @@ QString ComputerControlListModel::computerStateDescription( const ComputerContro
 
 QString ComputerControlListModel::loggedOnUserInformation( const ComputerControlInterface::Pointer& controlInterface )
 {
-	if( controlInterface->state() == ComputerControlInterface::Connected )
+	if( controlInterface->state() == ComputerControlInterface::State::Connected )
 	{
-		if( controlInterface->user().isEmpty() )
+		if( controlInterface->userLoginName().isEmpty() )
 		{
 			return tr( "No user logged on" );
 		}
 
-		return tr( "Logged on user: %1" ).arg( controlInterface->user() );
+		auto user = controlInterface->userLoginName();
+		if( controlInterface->userFullName().isEmpty() == false )
+		{
+			user = QStringLiteral( "%1 (%2)" ).arg( user, controlInterface->userFullName() );
+		}
+
+		return tr( "Logged on user: %1" ).arg( user );
 	}
 
-	return QString();
+	return {};
 }
 
 
@@ -444,7 +486,7 @@ QString ComputerControlListModel::activeFeatures( const ComputerControlInterface
 		}
 	}
 
-	featureNames.removeAll( QString() );
+	featureNames.removeAll( {} );
 
 	return featureNames.join( QStringLiteral(", ") );
 }
