@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2017-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -22,30 +22,15 @@
  *
  */
 
-#ifndef BUILTIN_DIRECTORY_CONFIGURATION_H
-#define BUILTIN_DIRECTORY_CONFIGURATION_H
+#pragma once
+
+#include <QJsonArray>
 
 #include "Configuration/Proxy.h"
 
 #define FOREACH_BUILTIN_DIRECTORY_CONFIG_PROPERTY(OP) \
-	OP( BuiltinDirectoryConfiguration, m_configuration, JSONARRAY, networkObjects, setNetworkObjects, "NetworkObjects", "BuiltinDirectory" );	\
+	OP( BuiltinDirectoryConfiguration, m_configuration, QJsonArray, networkObjects, setNetworkObjects, "NetworkObjects", "BuiltinDirectory", QJsonArray(), Configuration::Property::Flag::Standard )	\
 	/* legacy properties required for upgrade */ \
-	OP( BuiltinDirectoryConfiguration, m_configuration, JSONARRAY, localDataNetworkObjects, setLocalDataNetworkObjects, "NetworkObjects", "LocalData" );	\
+	OP( BuiltinDirectoryConfiguration, m_configuration, QJsonArray, legacyLocalDataNetworkObjects, setLegacyLocalDataNetworkObjects, "NetworkObjects", "LocalData", QJsonArray(), Configuration::Property::Flag::Legacy )	\
 
-// clazy:excludeall=ctor-missing-parent-argument
-
-class BuiltinDirectoryConfiguration : public Configuration::Proxy
-{
-	Q_OBJECT
-public:
-	BuiltinDirectoryConfiguration();
-
-	FOREACH_BUILTIN_DIRECTORY_CONFIG_PROPERTY(DECLARE_CONFIG_PROPERTY)
-
-public slots:
-	void setNetworkObjects( const QJsonArray& );
-	void setLocalDataNetworkObjects( const QJsonArray& );
-
-} ;
-
-#endif
+DECLARE_CONFIG_PROXY(BuiltinDirectoryConfiguration, FOREACH_BUILTIN_DIRECTORY_CONFIG_PROPERTY)
