@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2006-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -23,8 +23,7 @@
  *
  */
 
-#ifndef DEMO_SERVER_H
-#define DEMO_SERVER_H
+#pragma once
 
 #include <QElapsedTimer>
 #include <QReadWriteLock>
@@ -39,7 +38,7 @@ class DemoServer : public QObject
 {
 	Q_OBJECT
 public:
-	typedef QVector<QByteArray> MessageList;
+	using MessageList = QVector<QByteArray>;
 
 	DemoServer( int vncServerPort, const QString& vncServerPassword, const QString& demoAccessToken,
 				const DemoConfiguration& configuration, QObject *parent );
@@ -55,10 +54,7 @@ public:
 		return m_vncClientProtocol.serverInitMessage();
 	}
 
-	void lockDataForRead()
-	{
-		m_dataLock.lockForRead();
-	}
+	void lockDataForRead();
 
 	void unlockData()
 	{
@@ -75,13 +71,12 @@ public:
 		return m_framebufferUpdateMessages;
 	}
 
-private slots:
+private:
 	void acceptPendingConnections();
 	void reconnectToVncServer();
 	void readFromVncServer();
 	void requestFramebufferUpdate();
 
-private:
 	bool receiveVncServerMessage();
 	void enqueueFramebufferUpdateMessage( const QByteArray& message );
 
@@ -92,6 +87,8 @@ private:
 	bool setVncServerEncodings();
 
 	const DemoConfiguration& m_configuration;
+	const qint64 m_memoryLimit;
+	const int m_keyFrameInterval;
 	const int m_vncServerPort;
 	const QString m_demoAccessToken;
 
@@ -109,5 +106,3 @@ private:
 	MessageList m_framebufferUpdateMessages;
 
 } ;
-
-#endif
