@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2017-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -24,16 +24,17 @@
 
 #include "CommandLineIO.h"
 #include "ConfigurationManager.h"
-#include "VeyonConfiguration.h"
 #include "LdapNetworkObjectDirectory.h"
 #include "LdapPlugin.h"
 #include "LdapConfigurationPage.h"
 #include "LdapDirectory.h"
+#include "VeyonConfiguration.h"
 
 
 LdapPlugin::LdapPlugin( QObject* parent ) :
 	QObject( parent ),
-	m_configuration(),
+	m_configuration( &VeyonCore::config() ),
+	m_ldapClient( nullptr ),
 	m_ldapDirectory( nullptr ),
 	m_commands( {
 { QStringLiteral("autoconfigurebasedn"), tr( "Auto-configure the base DN via naming context" ) },
@@ -49,6 +50,9 @@ LdapPlugin::~LdapPlugin()
 {
 	delete m_ldapDirectory;
 	m_ldapDirectory = nullptr;
+
+	delete m_ldapClient;
+	m_ldapClient = nullptr;
 }
 
 
@@ -59,7 +63,7 @@ void LdapPlugin::upgrade( const QVersionNumber& oldVersion )
 	{
 		const auto upgradeFilter = []( const QString& filter ) {
 			if( filter.isEmpty() == false &&
-					filter.startsWith( '(' ) == false )
+					filter.startsWith( QLatin1Char('(') ) == false )
 			{
 				return QStringLiteral("(%1)").arg( filter );
 			}
@@ -74,12 +78,21 @@ void LdapPlugin::upgrade( const QVersionNumber& oldVersion )
 		m_configuration.setUsersFilter( upgradeFilter( m_configuration.usersFilter() ) );
 
 		// bind password not encrypted yet?
-		if( m_configuration.bindPasswordPlain().size() < MaximumPlaintextPasswordLength )
+		const auto rawBindPassword = m_configuration.bindPasswordProperty().variantValue().toString();
+		if( rawBindPassword.size() < MaximumPlaintextPasswordLength )
 		{
 			// setting it again will encrypt it
-			m_configuration.setBindPassword( m_configuration.bindPasswordPlain() );
+			m_configuration.setBindPassword( Configuration::Password::fromPlainText( rawBindPassword ) );
 		}
 	}
+	else if( oldVersion < QVersionNumber( 1, 2 ) )
+	{
+		m_configuration.setUserLoginNameAttribute( m_configuration.legacyUserLoginAttribute() );
+		m_configuration.setComputerLocationsByAttribute( m_configuration.legacyComputerRoomMembersByAttribute() );
+		m_configuration.setComputerLocationsByContainer( m_configuration.legacyComputerRoomMembersByContainer() );
+		m_configuration.setComputerLocationAttribute( m_configuration.legacyComputerRoomAttribute() );
+		m_configuration.setLocationNameAttribute( m_configuration.legacyComputerRoomNameAttribute() );
+	}
 }
 
 
@@ -117,7 +130,7 @@ QStringList LdapPlugin::userGroups( bool queryDomainGroups )
 {
 	Q_UNUSED(queryDomainGroups);
 
-	return ldapDirectory().toRelativeDnList( ldapDirectory().userGroups() );
+	return LdapClient::stripBaseDn( ldapDirectory().userGroups(), ldapClient().baseDn() );
 }
 
 
@@ -132,11 +145,11 @@ QStringList LdapPlugin::groupsOfUser( const QString& username, bool queryDomainG
 
 	if( userDn.isEmpty() )
 	{
-		qWarning() << "LdapPlugin::groupsOfUser(): empty user DN for user" << strippedUsername;
+		vWarning() << "empty user DN for user" << strippedUsername;
 		return QStringList();
 	}
 
-	return ldapDirectory().toRelativeDnList( ldapDirectory().groupsOfUser( userDn ) );
+	return LdapClient::stripBaseDn( ldapDirectory().groupsOfUser( userDn ), ldapClient().baseDn() );
 }
 
 
@@ -155,7 +168,8 @@ CommandLinePluginInterface::RunResult LdapPlugin::handle_autoconfigurebasedn( co
 
 	if( ldapUrl.isValid() == false || ldapUrl.host().isEmpty() )
 	{
-		qCritical() << "Please specify a valid LDAP url following the schema \"ldap[s]://[user[:password]@]hostname[:port]\"";
+		CommandLineIO::error( tr("Please specify a valid LDAP url following the schema "
+								 "\"ldap[s]://[user[:password]@]hostname[:port]\"") );
 		return InvalidArguments;
 	}
 
@@ -163,23 +177,23 @@ CommandLinePluginInterface::RunResult LdapPlugin::handle_autoconfigurebasedn( co
 
 	if( namingContextAttribute.isEmpty() )
 	{
-		qWarning( "No naming context attribute name given - falling back to configured value." );
+		CommandLineIO::warning( tr("No naming context attribute name given - falling back to configured value." ) );
 	}
 	else
 	{
 		m_configuration.setNamingContextAttribute( namingContextAttribute );
 	}
 
-	LdapDirectory ldapDirectory( m_configuration, ldapUrl );
-	QString baseDn = ldapDirectory.queryNamingContext();
+	LdapClient ldapClient( m_configuration, ldapUrl );
+	QString baseDn = ldapClient.queryNamingContexts().value( 0 );
 
 	if( baseDn.isEmpty() )
 	{
-		qCritical( "Could not query base DN. Please check your LDAP configuration." );
+		CommandLineIO::error( tr("Could not query base DN. Please check your LDAP configuration." ) );
 		return Failed;
 	}
 
-	qInfo() << "Configuring" << baseDn << "as base DN and disabling naming context queries.";
+	CommandLineIO::info( tr( "Configuring %1 as base DN and disabling naming context queries." ).arg( baseDn ) );
 
 	m_configuration.setBaseDn( baseDn );
 	m_configuration.setQueryNamingContext( false );
@@ -203,19 +217,19 @@ CommandLinePluginInterface::RunResult LdapPlugin::handle_query( const QStringLis
 	QString filter = arguments.value( 1 );
 	QStringList results;
 
-	if( objectType == QStringLiteral("rooms") )
+	if( objectType == QLatin1String("locations") )
 	{
-		results = ldapDirectory().computerRooms( filter );
+		results = ldapDirectory().computerLocations( filter );
 	}
-	else if( objectType == QStringLiteral("computers") )
+	else if( objectType == QLatin1String("computers") )
 	{
-		results = ldapDirectory().computers( filter );
+		results = ldapDirectory().computersByHostName( filter );
 	}
-	else if( objectType == QStringLiteral("groups") )
+	else if( objectType == QLatin1String("groups") )
 	{
 		results = ldapDirectory().groups( filter );
 	}
-	else if( objectType == QStringLiteral("users") )
+	else if( objectType == QLatin1String("users") )
 	{
 		results = ldapDirectory().users( filter );
 	}
@@ -239,7 +253,7 @@ CommandLinePluginInterface::RunResult LdapPlugin::handle_help( const QStringList
 {
 	QString command = arguments.value( 0 );
 
-	if( command == QStringLiteral("autoconfigurebasedn") )
+	if( command == QLatin1String("autoconfigurebasedn") )
 	{
 		printf( "\n"
 				"ldap autoconfigurebasedn <LDAP URL> [<naming context attribute name>]\n"
@@ -251,13 +265,13 @@ CommandLinePluginInterface::RunResult LdapPlugin::handle_help( const QStringList
 				"  ldap[s]://[user[:password]@]hostname[:port]\n\n" );
 		return NoResult;
 	}
-	else if( command == QStringLiteral("query") )
+	else if( command == QLatin1String("query") )
 	{
 		printf( "\n"
 				"ldap query <object type> [filter]\n"
 				"\n"
 				"Query objects from configured LDAP directory where <object type> may be one\n"
-				"of \"rooms\", \"computers\", \"groups\" or \"users\". You can optionally\n"
+				"of \"locations\", \"computers\", \"groups\" or \"users\". You can optionally\n"
 				"specify a filter such as \"foo*\".\n"
 				"\n" );
 		return NoResult;
@@ -268,6 +282,20 @@ CommandLinePluginInterface::RunResult LdapPlugin::handle_help( const QStringList
 
 
 
+LdapClient& LdapPlugin::ldapClient()
+{
+	if( m_ldapClient == nullptr )
+	{
+		m_ldapClient = new LdapClient( m_configuration );
+	}
+
+	// TODO: check whether still connected and reconnect if neccessary
+
+	return *m_ldapClient;
+}
+
+
+
 LdapDirectory& LdapPlugin::ldapDirectory()
 {
 	if( m_ldapDirectory == nullptr )
