@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2017-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -22,8 +22,7 @@
  *
  */
 
-#ifndef LINUX_CORE_FUNCTIONS_H
-#define LINUX_CORE_FUNCTIONS_H
+#pragma once
 
 #include <QDBusInterface>
 
@@ -36,17 +35,21 @@ class LinuxCoreFunctions : public PlatformCoreFunctions
 public:
 	LinuxCoreFunctions();
 
+	bool applyConfiguration() override;
+
 	void initNativeLoggingSystem( const QString& appName ) override;
 	void writeToNativeLoggingSystem( const QString& message, Logger::LogLevel loglevel ) override;
 
 	void reboot() override;
-	void powerDown() override;
+	void powerDown( bool installUpdates ) override;
 
 	void raiseWindow( QWidget* widget ) override;
 
 	void disableScreenSaver() override;
 	void restoreScreenSaverSettings() override;
 
+	void setSystemUiState( bool enabled ) override;
+
 	QString activeDesktopName() override;
 
 	bool isRunningAsAdmin() const override;
@@ -54,11 +57,11 @@ public:
 
 	bool runProgramAsUser( const QString& program, const QStringList& parameters,
 						   const QString& username,
-						   const QString& desktop = QString() ) override;
+						   const QString& desktop = {} ) override;
 
 	QString genericUrlHandler() const override;
 
-	typedef QSharedPointer<QDBusInterface> DBusInterfacePointer;
+	using DBusInterfacePointer = QSharedPointer<QDBusInterface>;
 
 	static DBusInterfacePointer kdeSessionManager();
 	static DBusInterfacePointer gnomeSessionManager();
@@ -76,5 +79,3 @@ private:
 	unsigned short m_dpmsOffTimeout;
 
 };
-
-#endif // LINUX_CORE_FUNCTIONS_H
