@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2017-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -22,8 +22,7 @@
  *
  */
 
-#ifndef EXTERNAL_VNC_SERVER_H
-#define EXTERNAL_VNC_SERVER_H
+#pragma once
 
 #include "PluginInterface.h"
 #include "VncServerPluginInterface.h"
@@ -35,8 +34,7 @@ class ExternalVncServer : public QObject, VncServerPluginInterface, PluginInterf
 	Q_PLUGIN_METADATA(IID "io.veyon.Veyon.Plugins.ExternalVncServer")
 	Q_INTERFACES(PluginInterface VncServerPluginInterface)
 public:
-	ExternalVncServer( QObject* parent = nullptr );
-	~ExternalVncServer() override;
+	explicit ExternalVncServer( QObject* parent = nullptr );
 
 	Plugin::Uid uid() const override
 	{
@@ -93,5 +91,3 @@ private:
 	ExternalVncServerConfiguration m_configuration;
 
 };
-
-#endif // EXTERNAL_VNC_SERVER_H
