@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2017-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -28,6 +28,7 @@
 #include <QThread>
 
 #include "BuiltinX11VncServer.h"
+#include "VeyonConfiguration.h"
 #include "X11VncConfigurationWidget.h"
 
 extern "C" int x11vnc_main( int argc, char * * argv );
@@ -35,13 +36,7 @@ extern "C" int x11vnc_main( int argc, char * * argv );
 
 BuiltinX11VncServer::BuiltinX11VncServer( QObject* parent ) :
 	QObject( parent ),
-	m_configuration()
-{
-}
-
-
-
-BuiltinX11VncServer::~BuiltinX11VncServer()
+	m_configuration( &VeyonCore::config() )
 {
 }
 
@@ -62,17 +57,19 @@ void BuiltinX11VncServer::prepareServer()
 
 void BuiltinX11VncServer::runServer( int serverPort, const QString& password )
 {
-	QStringList cmdline = { "-localhost",
-							"-nosel",			// do not exchange clipboard-contents
-							"-nosetclipboard",	// do not exchange clipboard-contents
-							"-rfbport", QString::number( serverPort ) // set port at which the VNC server should listen
+	QStringList cmdline = { QStringLiteral("-localhost"),
+							QStringLiteral("-nosel"),			// do not exchange clipboard-contents
+							QStringLiteral("-nosetclipboard"),	// do not exchange clipboard-contents
+							QStringLiteral("-rfbport"), QString::number( serverPort ), // set port at which the VNC server should listen
+							QStringLiteral("-rfbportv6"), QString::number( serverPort ), // set IPv6 port at which the VNC server should listen
+							QStringLiteral("-no6"),
 						  } ;
 
 	const auto extraArguments = m_configuration.extraArguments();
 
 	if( extraArguments.isEmpty() == false )
 	{
-		cmdline.append( extraArguments.split( ' ' ) );
+		cmdline.append( extraArguments.split( QLatin1Char(' ') ) );
 	}
 
 	if( m_configuration.isXDamageDisabled() )
@@ -98,7 +95,7 @@ void BuiltinX11VncServer::runServer( int serverPort, const QString& password )
 	QTemporaryFile tempFile;
 	if( tempFile.open() == false ) // Flawfinder: ignore
 	{
-		qCritical() << Q_FUNC_INFO << "Could not create temporary file!";
+		vCritical() << "Could not create temporary file!";
 		return;
 	}
 	tempFile.write( password.toLocal8Bit() );
@@ -116,8 +113,8 @@ void BuiltinX11VncServer::runServer( int serverPort, const QString& password )
 	x11vnc.start( QStringLiteral("x11vnc"), cmdline );
 	if( x11vnc.waitForStarted() == false )
 	{
-		qCritical() << "Could not start external x11vnc:" << x11vnc.errorString();
-		qCritical() << "Please make sure x11vnc is installed and installation directory is in PATH!";
+		vCritical() << "Could not start external x11vnc:" << x11vnc.errorString();
+		vCritical() << "Please make sure x11vnc is installed and installation directory is in PATH!";
 		QThread::msleep( 5000 );
 	}
 	else
@@ -125,7 +122,7 @@ void BuiltinX11VncServer::runServer( int serverPort, const QString& password )
 		x11vnc.waitForFinished( -1 );
 	}
 #else
-	cmdline.append( { "-passwd", password } );
+	cmdline.append( { QStringLiteral("-passwd"), password } );
 
 	// build new C-style command line array based on cmdline-QStringList
 	const auto appArguments = QCoreApplication::arguments();
@@ -137,7 +134,7 @@ void BuiltinX11VncServer::runServer( int serverPort, const QString& password )
 	{
 		const auto len = static_cast<size_t>( it->length() );
 		argv[argc] = new char[len + 1];
-		strncpy( argv[argc], it->toUtf8().constData(), len );
+		strncpy( argv[argc], it->toUtf8().constData(), len ); // Flawfinder: ignore
 		argv[argc][len] = 0;
 	}
 
@@ -152,3 +149,6 @@ void BuiltinX11VncServer::runServer( int serverPort, const QString& password )
 	delete[] argv;
 #endif
 }
+
+
+IMPLEMENT_CONFIG_PROXY(X11VncConfiguration)
