@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2017-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -22,8 +22,7 @@
  *
  */
 
-#ifndef BUILTIN_X11VNC_SERVER_H
-#define BUILTIN_X11VNC_SERVER_H
+#pragma once
 
 #include "PluginInterface.h"
 #include "VncServerPluginInterface.h"
@@ -35,8 +34,7 @@ class BuiltinX11VncServer : public QObject, VncServerPluginInterface, PluginInte
 	Q_PLUGIN_METADATA(IID "io.veyon.Veyon.Plugins.BuiltinX11VncServer")
 	Q_INTERFACES(PluginInterface VncServerPluginInterface)
 public:
-	BuiltinX11VncServer( QObject* parent = nullptr );
-	~BuiltinX11VncServer() override;
+	explicit BuiltinX11VncServer( QObject* parent = nullptr );
 
 	Plugin::Uid uid() const override
 	{
@@ -86,12 +84,10 @@ public:
 
 	QString configuredPassword() override
 	{
-		return QString();
+		return {};
 	}
 
 private:
 	X11VncConfiguration m_configuration;
 
 };
-
-#endif // BUILTIN_X11VNC_SERVER_H
