@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2006-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -23,6 +23,8 @@
  *
  */
 
+#include "rfb/rfbproto.h"
+
 #include "AuthenticationCredentials.h"
 #include "CryptoCore.h"
 #include "VeyonConfiguration.h"
@@ -30,15 +32,13 @@
 #include "VncServer.h"
 #include "VncServerPluginInterface.h"
 
-#include "rfb/rfbproto.h"
-
 
 VncServer::VncServer( QObject* parent ) :
 	QThread( parent ),
 	m_pluginInterface( nullptr )
 {
 	VeyonCore::authenticationCredentials().setInternalVncServerPassword(
-				CryptoCore::generateChallenge().toBase64().left( MAXPWLEN ) );
+				QString::fromLatin1( CryptoCore::generateChallenge().toBase64().left( MAXPWLEN ) ) );
 
 	VncServerPluginInterfaceList defaultVncServerPlugins;
 
@@ -64,7 +64,7 @@ VncServer::VncServer( QObject* parent ) :
 	{
 		if( defaultVncServerPlugins.isEmpty() )
 		{
-			qCritical( "VncServer::VncServer(): no VNC server plugins found!" );
+			vCritical() << "no VNC server plugins found!";
 		}
 		else
 		{
@@ -77,14 +77,14 @@ VncServer::VncServer( QObject* parent ) :
 
 VncServer::~VncServer()
 {
-	qDebug(Q_FUNC_INFO);
+	vDebug();
 }
 
 
 
 void VncServer::prepare()
 {
-	qDebug(Q_FUNC_INFO);
+	vDebug();
 
 	if( m_pluginInterface )
 	{
@@ -122,7 +122,7 @@ void VncServer::run()
 {
 	if( m_pluginInterface )
 	{
-		qDebug() << Q_FUNC_INFO << "running";
+		vDebug() << "running";
 
 		if( m_pluginInterface->configuredServerPort() > 0 )
 		{
@@ -136,6 +136,6 @@ void VncServer::run()
 
 		m_pluginInterface->runServer( serverPort(), password() );
 
-		qDebug() << Q_FUNC_INFO << "finished";
+		vDebug() << "finished";
 	}
 }
