@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2006-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -23,8 +23,7 @@
  *
  */
 
-#ifndef VNC_SERVER_H
-#define VNC_SERVER_H
+#pragma once
 
 #include <QThread>
 
@@ -34,8 +33,8 @@ class VncServer : public QThread
 {
 	Q_OBJECT
 public:
-	VncServer( QObject* parent = nullptr );
-	virtual ~VncServer();
+	explicit VncServer( QObject* parent = nullptr );
+	~VncServer() override;
 
 	void prepare();
 
@@ -44,10 +43,8 @@ public:
 	QString password() const;
 
 private:
-	virtual void run();
+	void run() override;
 
 	VncServerPluginInterface* m_pluginInterface;
 
 } ;
-
-#endif
