@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2006-2019 Tobias Junghans <tobydox@veyon.io>
  *
- * This file is part of Veyon - http://veyon.io
+ * This file is part of Veyon - https://veyon.io
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -34,21 +34,12 @@ int main( int argc, char **argv )
 
 	VeyonCore core( &app, QStringLiteral("Server") );
 
-	auto server = new ComputerControlServer;
-	if( server->start() == false )
+	ComputerControlServer server( &core );
+	if( server.start() == false )
 	{
-		qInfo( "Failed to start server" );
-		delete server;
+		vCritical() << "Failed to start server";
 		return -1;
 	}
 
-	qInfo( "Exec" );
-
-	int ret = app.exec();
-
-	delete server;
-
-	qInfo( "Exec Done" );
-
-	return ret;
+	return core.exec();
 }
